@@ -17,11 +17,12 @@ module Snap.Core
   , finishWith
   , catchFinishWith
   , pass
-  , terminateConnection
 
     -- *** Escaping HTTP
   , EscapeHttpHandler
+  , EscapeSnap(..)
   , escapeHttp
+  , terminateConnection
 
     -- ** Routing
   , method
@@ -51,7 +52,6 @@ module Snap.Core
 
     -- ** Grabbing/transforming request bodies
   , runRequestBody
-  , getRequestBody
   , readRequestBody
   , transformRequestBody
 
@@ -71,18 +71,18 @@ module Snap.Core
   , addHeader
   , setHeader
   , getHeader
-  , getHeaders
   , listHeaders
   , deleteHeader
   , ipHeaderFilter
   , ipHeaderFilter'
 
     -- ** Requests
-  , rqServerName
+  , rqHeaders
+  , rqHostName
+  , rqClientAddr
+  , rqClientPort
+  , rqServerAddr
   , rqServerPort
-  , rqRemoteAddr
-  , rqRemotePort
-  , rqLocalAddr
   , rqLocalHostname
   , rqIsSecure
   , rqContentLength
@@ -108,6 +108,10 @@ module Snap.Core
   , rqModifyParams
   , rqSetParam
 
+    -- *** Deprecated functions
+  , rqRemoteAddr
+  , rqRemotePort
+
     -- ** Responses
   , emptyResponse
   , setResponseCode
@@ -127,8 +131,6 @@ module Snap.Core
   , clearContentLength
   , redirect
   , redirect'
-  , setBufferingMode
-  , getBufferingMode
 
     -- *** Response I/O
   , setResponseBody
@@ -146,13 +148,8 @@ module Snap.Core
   , setTimeout
   , extendTimeout
   , modifyTimeout
-  , getTimeoutAction
   , getTimeoutModifier
 
-    -- * Iteratee
-  , Enumerator
-  , SomeEnumerator(..)
-
     -- * HTTP utilities
   , formatHttpTime
   , parseHttpTime
@@ -165,14 +162,12 @@ module Snap.Core
   ) where
 
 ------------------------------------------------------------------------------
-import           Snap.Internal.Exceptions (EscapeHttpHandler)
-import           Snap.Internal.Http.Types
-import           Snap.Internal.Instances ()
-import           Snap.Internal.Parsing
-import           Snap.Internal.Routing
-import           Snap.Internal.Types
-import           Snap.Iteratee (Enumerator)
-import           Snap.Types.Headers (Headers)
+import           Snap.Internal.Core       (EscapeHttpHandler, EscapeSnap (..), MonadSnap (..), NoHandlerException (..), Snap, addToOutput, bracketSnap, catchFinishWith, dir, escapeHttp, expireCookie, extendTimeout, finishWith, getCookie, getParam, getParams, getPostParam, getPostParams, getQueryParam, getQueryParams, getRequest, getResponse, getTimeoutModifier, getsRequest, getsResponse, ifTop, ipHeaderFilter, ipHeaderFilter', localRequest, logError, method, methods, modifyRequest, modifyResponse, modifyTimeout, pass, path, pathArg, putRequest, putResponse, readCookie, readRequestBody, redirect, redirect', runRequestBody, runSnap, sendFile, sendFilePartial, setTimeout, terminateConnection, transformRequestBody, withRequest, withResponse, writeBS, writeBuilder, writeLBS, writeLazyText, writeText)
+import           Snap.Internal.Http.Types (Cookie (..), HasHeaders (..), HttpVersion, Method (..), Params, Request (rqClientAddr, rqClientPort, rqContentLength, rqContextPath, rqCookies, rqHeaders, rqHostName, rqIsSecure, rqLocalHostname, rqMethod, rqParams, rqPathInfo, rqPostParams, rqQueryParams, rqQueryString, rqServerAddr, rqServerPort, rqURI, rqVersion), Response (rspStatus, rspStatusReason), addHeader, addResponseCookie, clearContentLength, deleteHeader, deleteResponseCookie, emptyResponse, formatHttpTime, getHeader, getResponseCookie, getResponseCookies, listHeaders, modifyResponseBody, modifyResponseCookie, parseHttpTime, rqModifyParams, rqParam, rqPostParam, rqQueryParam, rqRemoteAddr, rqRemotePort, rqSetParam, setContentLength, setContentType, setHeader, setResponseBody, setResponseCode, setResponseStatus)
+import           Snap.Internal.Instances  ()
+import           Snap.Internal.Parsing    (buildUrlEncoded, parseUrlEncoded, printUrlEncoded, urlDecode, urlEncode, urlEncodeBuilder)
+import           Snap.Internal.Routing    (route, routeLocal)
+import           Snap.Types.Headers       (Headers)
 ------------------------------------------------------------------------------
 
 -- $httpDoc
