@@ -29,6 +29,7 @@ module Snap.Test
   , addHeader
   , setContentType
   , setHeader
+  , addCookies
   , setHttpVersion
   , setQueryString
   , setQueryStringRaw
@@ -46,13 +47,11 @@ module Snap.Test
    -- * Getting response bodies
   , getResponseBody
 
-   -- * Dumping HTTP Responses
-  , dumpResponse
+   -- * Dumping HTTP Messages
+  , requestToString
   , responseToString
   )
   where
 
-import           Snap.Internal.Test.Assertions
-import           Snap.Internal.Test.RequestBuilder
-
-
+import           Snap.Internal.Test.Assertions     (assert404, assertBodyContains, assertRedirect, assertRedirectTo, assertSuccess, getResponseBody)
+import           Snap.Internal.Test.RequestBuilder (FileData (..), MultipartParam (..), MultipartParams, RequestBuilder, RequestType (..), addCookies, addHeader, buildRequest, delete, evalHandler, evalHandlerM, get, postMultipart, postRaw, postUrlEncoded, put, requestToString, responseToString, runHandler, runHandlerM, setContentType, setHeader, setHttpVersion, setQueryString, setQueryStringRaw, setRequestPath, setRequestType, setSecure)
